package com.caimei.controller;


import com.alibaba.fastjson.JSONObject;
import com.caimei.model.ResponseJson;
import com.caimei.service.OrderService;
import com.caimei.utils.KeyUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Aslee
 */
@Slf4j
@RestController
@RequestMapping("/order")
public class OrderApi {
    private OrderService orderService;

    @Autowired
    public void setOrderService(OrderService orderService) {
        this.orderService = orderService;
    }

    /**
     * 订单详情
     */
    @PostMapping("/detail")
    public ResponseJson<Map<String, Object>> orderDetail(HttpServletRequest request) {
        String cmAccount = request.getParameter("cmAccount");
        String data = request.getParameter("data");
        if (StringUtils.isEmpty(cmAccount)) {
            return ResponseJson.error("采美账户不能为空", null);
        }
        if (StringUtils.isEmpty(data)) {
            return ResponseJson.error("订单数据不能为空", null);
        }
        return orderService.orderDetail(cmAccount, data);
    }

    /**
     * 取消订单
     */
    @PostMapping("/cancel")
    public ResponseJson orderCancel(HttpServletRequest request) {
        String cmAccount = request.getParameter("cmAccount");
        String data = request.getParameter("data");
        if (StringUtils.isEmpty(cmAccount)) {
            return ResponseJson.error("采美账户不能为空", null);
        }
        if (StringUtils.isEmpty(data)) {
            return ResponseJson.error("订单数据不能为空", null);
        }
        return orderService.orderCancel(cmAccount, data);
    }

    /**
     * 余额抵扣
     */
    @PostMapping("/balance/deduction")
    public ResponseJson<Map<String, Object>> balanceDeduction(HttpServletRequest request) {
        String cmAccount = request.getParameter("cmAccount");
        String data = request.getParameter("data");
        if (StringUtils.isEmpty(cmAccount)) {
            return ResponseJson.error("采美账户不能为空", null);
        }
        if (StringUtils.isEmpty(data)) {
            return ResponseJson.error("订单数据不能为空", null);
        }
        return orderService.balanceDeduction(cmAccount, data);
    }

    /**
     * 测试通知回调
     */
    @PostMapping("/pay/callBack")
    public ResponseJson callBack(HttpServletRequest request) {
        String cmAccount = request.getParameter("cmAccount");
        String data = request.getParameter("data");
        if (StringUtils.isEmpty(cmAccount)) {
            return ResponseJson.error("采美账户不能为空", null);
        }
        if (StringUtils.isEmpty(data)) {
            return ResponseJson.error("订单数据不能为空", null);
        }
        return orderService.payCallBack(cmAccount, data);
    }
}
