package com.caimei.controller;


import com.caimei.model.ResponseJson;
import com.caimei.service.OrderService;
import com.caimei.service.OrderSubmitService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

/**
 * @author Aslee
 */
@Slf4j
@RestController
@RequestMapping("/order")
public class OrderSubmitApi {
    private OrderSubmitService orderSubmitService;

    @Autowired
    public void setOrderSubmitService(OrderSubmitService orderSubmitService) {
        this.orderSubmitService = orderSubmitService;
    }



    /**
     * 下单
     * data 参数格式：{
     *                     "sign": ****,        //签名
     *                     "notifyUrl": ****,       //订单支付完成通知回调地址
     *                     "addressInfo": {         //收货地址信息
     *                          "receiveMan":"收货人",     //收货人
     *                          "mobile":"15816666666",    //手机号
     *                          "provinceName":"广东省",   //省
     *                          "cityName":"深圳市",       //市/县
     *                          "townName":"福田区",       //区
     *                          "addressDetail":"*****"   //详细地址
     *                     },          //
     *                     "orderId": cm6858,          //对接组织的订单Id，由组织自行生成
     *                     "productInfo": { "0001":2,    // 商品Id:购买数量
     *                                       "0002":3
     *                                   },
*                          "payInfo": {
 *                                  "orderShouldPayFee": 609.11,     // 订单总金额
 *                                  "balancePayFlag": 0,    //是否使用余额抵扣，0否，1是
     *                          },
     *                     "invoiceInfo": {}// 发票信息： {"type": 0},// 不开发票
     *                                     {
     *                                         "type": 2,             // 要发票
     *                                         "invoiceTitle": "单位名称",
     *                                         "corporationTaxNum": "NSRSBM97897",
     *                                         "registeredAddress": "注册地址",
     *                                         "registeredPhone": "15814011616",
     *                                         "openBank": "开户银行",
     *                                         "bankAccountNo": "987987465465464"
     *                                     }
     *                 }
     */
    @PostMapping("/submit")
    public ResponseJson orderSubmit(HttpServletRequest request) {
        log.info("API对接组织下单");
        String cmAccount = request.getParameter("cmAccount");
        String data = request.getParameter("data");
        if (StringUtils.isEmpty(cmAccount)) {
            return ResponseJson.error("采美账户不能为空");
        }
        if (StringUtils.isEmpty(data)) {
            return ResponseJson.error("订单数据不能为空");
        }
        return orderSubmitService.orderSubmit(cmAccount, data);
    }
}
