package com.caimei.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.caimei.mapper.OrderMapper;
import com.caimei.mapper.OrganizeMapper;
import com.caimei.mapper.ProductMapper;
import com.caimei.model.ResponseJson;
import com.caimei.model.enumerate.ReceivablesType;
import com.caimei.model.po.*;
import com.caimei.model.vo.*;
import com.caimei.service.OrderService;
import com.caimei.service.OrderSubmitService;
import com.caimei.utils.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author Aslee
 */
@Slf4j
@Service
public class OrderServiceImpl implements OrderService {
    @Resource
    private OrderMapper orderMapper;
    @Resource
    private OrganizeMapper organizeMapper;
    @Resource
    private ProductMapper productMapper;

    @Value("${caimei.oldapi}")
    private String domain;

    private OrderSubmitService orderSubmitService;

    @Autowired
    public void setOrderSubmitService(OrderSubmitService orderSubmitService) {
        this.orderSubmitService = orderSubmitService;
    }

    @Override
    public ResponseJson<Map<String,Object>> orderDetail(String cmAccount, String data) {
        // 获取组织信息
        CmApiOrganizePo organizePo = organizeMapper.getOrganizeByCmAccount(cmAccount);
        if (null == organizePo) {
            return ResponseJson.error("参数异常，用户不存在", null);
        }
        // 获取用户id
        Integer userId = organizePo.getUserId();
        JSONObject orderInfo = null;
        try {
            // 使用组织对应的公钥解密，获得订单数据
            orderInfo = KeyUtils.decryptDataPublic(data, organizePo.getPublicKey());
            // 私钥验签
            String preSign = orderInfo.getString("sign");
            orderInfo.remove("sign");
            String sufSign = KeyUtils.buildSign(orderInfo, organizePo.getPrivateKey());
            if (!preSign.equals(sufSign)) {
                return ResponseJson.error("验签失败", null);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseJson.error("解密失败", null);
        }
        String apiOrganizeOrderId = (String) orderInfo.get("orderId");
        if (StringUtils.isEmpty(apiOrganizeOrderId)) {
            return ResponseJson.error("参数异常，订单Id不能为空",null);
        }
        // 订单信息,运费
        OrderVo order = orderMapper.getOrderInfo(userId, apiOrganizeOrderId);
        if (null == order) {
            return ResponseJson.error("该订单不存在", null);
        }
        // 收货地址
        AddressInfoVo addressInfo = orderMapper.getAddressInfo(order.getOrderID().intValue());
        // 收款列表
        getDiscernReceipt(order);
        CmOrderVo cmOrderVo = new CmOrderVo();
        // 应付金额
        cmOrderVo.setPayableAmount(order.getPayableAmount());
        // 待付金额
        cmOrderVo.setPendingPayments(order.getPendingPayments());
        // 收发状态
        cmOrderVo.setStatus(order.getStatus());
        // 退款状态
        cmOrderVo.setRefundType(order.getRefundType());
        // 退款记录
        List<CmReturnedPurchasePo> returnedPurchaseList = orderMapper.findReturnedPurchase(order.getOrderID());
        if (returnedPurchaseList != null && returnedPurchaseList.size() > 0) {
            BigDecimal returnedPurchaseFee = new BigDecimal(0);
            for (CmReturnedPurchasePo returnedPurchase : returnedPurchaseList) {
                returnedPurchaseFee = returnedPurchaseFee.add(returnedPurchase.getRefundFee());
            }
            //退款总金额
            cmOrderVo.setReturnedPurchaseFee(returnedPurchaseFee);
        }
        // 发票信息
        BpOrderInvoiceVo invoiceInfo = orderMapper.getOrderInvoiceVo(order.getOrderID());
        // 商品信息
        List<OrderProductVo> dbProductList = productMapper.getOrderProduct(order.getOrderID());
        List<DisplayOrderProductVo> orderProductList = new ArrayList<>();
        DecimalFormat decimalFormat = new DecimalFormat("0000");
        for (OrderProductVo orderProduct : dbProductList) {
            DisplayOrderProductVo product = new DisplayOrderProductVo();
            product.setProductId(decimalFormat.format(orderProduct.getApiOrganizeProductId()));
            product.setImage(ImageUtils.getImageURL("product", orderProduct.getProductImage(), 0, domain));
            product.setProductName(orderProduct.getName());
            product.setShopName(orderProduct.getShopName());
            product.setUnit(orderProduct.getProductUnit());
            product.setPrice(orderProduct.getDiscountPrice());
            product.setProductNum(orderProduct.getNum());
            product.setTaxRate(orderProduct.getTaxRate());
            //是否包含充值商品
            /*if (OrderNoUtils.getRechargeProduct(orderProduct.getProductID())) {
                order.setRechargeGoods(true);
            }*/
            //已发货数量
            product.setShipmentsNum(orderProduct.getNum() + orderProduct.getPresentNum() - orderProduct.getNotOutStore());
            //已退货数量
            Integer returnedNum = orderMapper.returnedPurchase(orderProduct.getShopOrderID(), orderProduct.getProductID());
            returnedNum = null != returnedNum ? returnedNum : 0;
            product.setReturnedNum(returnedNum);
            //已取消发货数量
            Integer actualCancelNum = orderMapper.actualCancelNum(orderProduct.getShopOrderID(), orderProduct.getProductID());
            actualCancelNum = null != actualCancelNum ? actualCancelNum : 0;
            product.setActualCancelNum(actualCancelNum);
            //判断商品价格是否含税
            if ("1".equals(orderProduct.getIncludedTax()) || ("0".equals(orderProduct.getIncludedTax()) && ("1".equals(orderProduct.getInvoiceType()) || "2".equals(orderProduct.getInvoiceType())))) {
                product.setIncludedTaxFlag("2");
            } else if (orderProduct.getIncludedTax() != null && "0".equals(orderProduct.getIncludedTax()) && "3".equals(orderProduct.getInvoiceType())) {
                product.setIncludedTaxFlag("1");
            } else {
                product.setIncludedTaxFlag("3");
            }
            //不含税可开票商品,单价/折后单价在原基础上加上税费
            if ("0".equals(orderProduct.getIncludedTax()) && ("1".equals(orderProduct.getInvoiceType()) || "2".equals(orderProduct.getInvoiceType()))) {
                product.setPrice(MathUtil.add(product.getPrice(), orderProduct.getAddedValueTax()));
            }
            orderProductList.add(product);
        }
        HashMap<String, Object> orderDetail = new HashMap<>();
        orderDetail.put("orderId", apiOrganizeOrderId);
        orderDetail.put("addressInfo", addressInfo);
        orderDetail.put("orderInfo", cmOrderVo);
        orderDetail.put("productInfo", orderProductList);
        orderDetail.put("invoiceInfo", invoiceInfo);
        return ResponseJson.success(orderDetail);
    }

    @Override
    public ResponseJson orderCancel(String cmAccount, String data) {
        // 获取组织信息
        CmApiOrganizePo organizePo = organizeMapper.getOrganizeByCmAccount(cmAccount);
        if (null == organizePo) {
            return ResponseJson.error("参数异常，用户不存在", null);
        }
        // 获取用户id
        Integer userId = organizePo.getUserId();
        JSONObject orderInfo = null;
        try {
            // 使用组织对应的公钥解密，获得订单数据
            orderInfo = KeyUtils.decryptDataPublic(data, organizePo.getPublicKey());
            // 私钥验签
            String preSign = orderInfo.getString("sign");
            orderInfo.remove("sign");
            String sufSign = KeyUtils.buildSign(orderInfo, organizePo.getPrivateKey());
            if (!preSign.equals(sufSign)) {
                return ResponseJson.error("验签失败", null);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseJson.error("解密失败", null);
        }
        String apiOrganizeOrderId = (String) orderInfo.get("orderId");
        if (StringUtils.isEmpty(apiOrganizeOrderId)) {
            return ResponseJson.error("参数异常，订单Id不能为空",null);
        }
        CmOrderPo order = orderMapper.getOrder(userId, apiOrganizeOrderId);
        if (null == order) {
            return ResponseJson.error("该订单不存在");
        }
        if (!("11".equals(order.getStatus()) || "0".equals(order.getStatus()))) {
            return ResponseJson.error("只有未收款未发货的订单才能取消");
        }
        order.setStatus("6");
        order.setCloseReason("协销或用户主动取消订单");
        order.setCloseTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));
        orderMapper.updateOrder(order);
        return ResponseJson.success();
    }

    /**
     * 余额抵扣
     * @param cmAccount
     * @param data
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseJson<Map<String, Object>> balanceDeduction(String cmAccount, String data) {
        // 获取组织信息
        CmApiOrganizePo organizePo = organizeMapper.getOrganizeByCmAccount(cmAccount);
        if (null == organizePo) {
            return ResponseJson.error("参数异常，用户不存在", null);
        }
        // 获取用户id
        Integer userId = organizePo.getUserId();
        UserPo user = orderMapper.getUserByUserId(userId);
        BigDecimal userMoney = user.getUserMoney();
        BigDecimal ableUserMoney = user.getAbleUserMoney();
        if (MathUtil.compare(ableUserMoney, BigDecimal.ZERO) == 0) {
            return ResponseJson.error("用户可用余额为0", null);
        }
        JSONObject orderInfo = null;
        try {
            // 使用组织对应的公钥解密，获得订单数据
            orderInfo = KeyUtils.decryptDataPublic(data, organizePo.getPublicKey());
            // 私钥验签
            String preSign = orderInfo.getString("sign");
            orderInfo.remove("sign");
            String sufSign = KeyUtils.buildSign(orderInfo, organizePo.getPrivateKey());
            if (!preSign.equals(sufSign)) {
                return ResponseJson.error("验签失败", null);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseJson.error("解密失败", null);
        }
        String apiOrganizeOrderId = (String) orderInfo.get("orderId");
        if (StringUtils.isEmpty(apiOrganizeOrderId)) {
            return ResponseJson.error("参数异常，订单Id不能为空",null);
        }
        OrderVo order = orderMapper.getOrderInfo(userId, apiOrganizeOrderId);
        if (null == order) {
            return ResponseJson.error("该订单不存在",null);
        }
        getDiscernReceipt(order);
        //本次余额抵扣金额
        BigDecimal balancePayFee = BigDecimal.ZERO;
        if (MathUtil.compare(ableUserMoney, 0) > 0) {
            if (MathUtil.compare(ableUserMoney, order.getPendingPayments()) < 0) {
                if (!"0".equals(order.getStatus())) {
                    //待确认状态,不修改订单状态
                    if ("1".equals(order.getSendOutStatus())) {
                        order.setStatus("21");
                    } else if ("2".equals(order.getSendOutStatus())) {
                        order.setStatus("22");
                    } else {
                        order.setStatus("23");
                    }
                }
                order.setReceiptStatus("2");
                balancePayFee = ableUserMoney;
            } else {
                if (!"0".equals(order.getStatus())) {
                    if ("1".equals(order.getSendOutStatus())) {
                        order.setStatus("31");
                    } else if ("2".equals(order.getSendOutStatus())) {
                        order.setStatus("32");
                    } else {
                        order.setStatus("33");
                    }
                }
                order.setReceiptStatus("3");
                balancePayFee = order.getPendingPayments();
            }
            order.setBalancePayFee(MathUtil.add(order.getBalancePayFee(), balancePayFee));
            order.setPayableAmount(MathUtil.sub(order.getPayableAmount(), balancePayFee));
            orderMapper.updateOrder(order);
            //剩余待付金额
            order.setPendingPayments(MathUtil.sub(order.getPendingPayments(), balancePayFee));
            //修改账户余额
            if (!"0".equals(order.getStatus())) {
                user.setUserMoney(MathUtil.sub(userMoney, balancePayFee));
                //保存收款记录
                orderSubmitService.saveDiscernReceipt(balancePayFee, order.getOrderID().intValue());
            }
            user.setAbleUserMoney(MathUtil.sub(ableUserMoney, balancePayFee));
            organizeMapper.updateMoney(user.getUserMoney(), user.getAbleUserMoney(), user.getUserID());
            //保存余额收支记录
            orderSubmitService.saveBalanceRecord(balancePayFee, order.getOrderID().intValue(), user.getUserID());
        }
        Map<String, Object> map = new HashMap<>(2);
        map.put("balancePayFee", balancePayFee);
        map.put("orderId", apiOrganizeOrderId);
        map.put("payableAmount", order.getPayableAmount());
        if (MathUtil.compare(order.getPayableAmount(),BigDecimal.ZERO) >0) {
            // 余额抵扣,部分付款
            map.put("code", "1");
            map.put("msg", "抵扣成功部分支付");
        } else {
            // 余额抵扣，完成付款
            map.put("code", "2");
            map.put("msg", "抵扣成功完成支付");
        }
        return ResponseJson.success(map);
    }

    @Override
    public ResponseJson payCallBack(String cmAccount, String data) {
        // 获取组织信息
        CmApiOrganizePo organizePo = organizeMapper.getOrganizeByCmAccount(cmAccount);
        if (null == organizePo) {
            return ResponseJson.error("参数异常，用户不存在", null);
        }
        // 获取用户id
        Integer userId = organizePo.getUserId();
        JSONObject orderInfo = null;
        try {
            // 使用组织对应的公钥解密，获得订单数据
            orderInfo = KeyUtils.decryptDataPublic(data, organizePo.getPublicKey());
            // 私钥验签
            String preSign = orderInfo.getString("sign");
            orderInfo.remove("sign");
            String sufSign = KeyUtils.buildSign(orderInfo, organizePo.getPrivateKey());
            if (!preSign.equals(sufSign)) {
                return ResponseJson.error("验签失败", null);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseJson.error("解密失败", null);
        }
        String orderId = (String) orderInfo.get("orderId");
        String result = (String) orderInfo.get("result");
        Double amount = orderInfo.getDouble("amount");
        log.info(">>>>>>>>>>>>>>>>>>>>>>>>>>>订单支付完成通知回调：orderId:" + orderId + ",amount:" + amount + ",result:" + result);
        return ResponseJson.success("SUCCESS");
    }





    private List<DiscernReceiptVo> getDiscernReceipt(OrderVo order) {
        //支付记录
        List<DiscernReceiptVo> discernReceiptList = orderMapper.findDiscernReceipt(order.getOrderID().intValue());
        BigDecimal receiptAmount = BigDecimal.ZERO;
        //订单款
        if (discernReceiptList != null && discernReceiptList.size() > 0) {
            for (DiscernReceiptVo discernReceipt : discernReceiptList) {
                receiptAmount = receiptAmount.add(discernReceipt.getAssociateAmount());
                if (discernReceipt.getPayType() != null) {
                    discernReceipt.setPayType(ReceivablesType.getReceivablesType(Integer.parseInt(discernReceipt.getPayType())));
                }
            }
        }
        order.setPendingPayments(MathUtil.sub(order.getPayableAmount(), receiptAmount));
        //支付总金额
        order.setReceiptAmount(receiptAmount);
        return discernReceiptList;
    }

}
